{-----------------------------------------------------------------------------
 Unit Name: SignalHandler
 Author:    hardy
 Date:      10-Jun-2006
 Purpose:   Signal-Handling fr Anwendung
            Fgt der Anwendung eine Signal-Queue hinzu und ermglicht
            beliebigen Objekten an der Signalverteilung teilzunehmen
 History:
-----------------------------------------------------------------------------}
unit SignalHandler;

{$MODE Delphi}

interface

uses
  SysUtils, Classes, ExtCtrls;

type
  TAppSignalStruct  = class;
  TSignalSlotStruct = class;

  TSignalSlot         = procedure(ASignal: TAppSignalStruct) of object;
  TSignalResultSlot   = procedure(ASender    : TObject;
                                  AResultSize: integer;
                                  var AResultBuf) of object;

  TAppSignalStruct = class(TObject)
    Signal    : string;
    Sender    : TObject;
    Enabled   : boolean;
    ResultSlot: TSignalResultSlot;
  end; // of TAppSignalStruct

  TSignalSlotStruct = class(TObject)
    Signal    : string;
    Receiver  : TObject;
    SignalSlot: TSignalSlot;
  end; // of TSignalSlotStruct

  TAppSignalHandler = class(TComponent)
  private
    FSignalSlotList: TList;
    FSignalList    : TList;

    function GetSignal(ASignal: string): TAppSignalStruct;
    function LookForSignalSlot
      (ASignal: string; AOwner: TObject): TSignalSlotStruct; overload;
    function LookForSignalSlot(ASignal: string): TSignalSlotStruct; overload;
    function LookForSignal
      (ASignal: string; AOwner: TObject): TAppSignalStruct;
    procedure EraseSignalSlots(ASignal: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;

    procedure AddSignalSlot(ASignal: string; ASlot: TSignalSlotStruct);
    procedure DelSignalSlot(ASignal: string; AOwner: TObject);
    function  AddNewSignal(ASignal: TAppSignalStruct): boolean;
    procedure DelSignal(ASignal: string; AOwner: TObject);
    procedure SendSignal(ASignal: string);
  end; // of TAppSignalHandler

var
  AppSignalHandler: TAppSignalHandler;


function CreateSignal(ASignal: string;
                      ASender: TObject;
                      AResultSlot: TSignalResultSlot): TAppSignalStruct;
implementation

{-----------------------------------------------------------------------------
  Function : CreateSignal
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string; ASender: TObject; AResultSlot: TSignalResultSlot
  Result:    TAppSignalStruct
-----------------------------------------------------------------------------}
function CreateSignal(ASignal: string;
                      ASender: TObject;
                      AResultSlot: TSignalResultSlot): TAppSignalStruct;
const
  cProcName = 'SignalHandler.CreateSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string; ASender: TObject; AResultSlot: TSignalResultSlot'
    // 'TAppSignalStruct';
  end; // of function SetErrorParams: string

begin
  { function body }
  if (ASignal <> '') and (ASender <> nil) then begin
    Result:= TAppSignalStruct.Create;
    Result.Signal := ASignal;
    Result.Sender := ASender;
    Result.Enabled:= true;
    Result.ResultSlot:= AResultSlot;
  end; // of if (ASignal <> '') and (ASender <> nil) then begin
end; // of CreateSignal

{*****************************************************************************}
{***                             TAppSignalHandler                           ***}
{*****************************************************************************}

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    Create
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: AOwner: TComponent
  Result:    None
-----------------------------------------------------------------------------}
constructor TAppSignalHandler.Create(AOwner: TComponent);
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.Create';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AOwner: TComponent'
    // 'None';
  end; // of function SetErrorParams: string

begin
  { function body }
  inherited Create(AOwner);
  FSignalSlotList:= TList.Create;
  FSignalList    := TList.Create;
end; // of TAppSignalHandler.Create

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    Destroy
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: None
  Result:    None
-----------------------------------------------------------------------------}
destructor TAppSignalHandler.Destroy;
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.Destroy';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'None'
    // 'None';
  end; // of function SetErrorParams: string

begin
  { function body }
  FSignalSlotList.Free;
  FSignalList.Free;
  inherited;
end; // of TAppSignalHandler.Destroy

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    GetSignal
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: ASignal: string
  Result:    TAppSignalStruct
-----------------------------------------------------------------------------}
function TAppSignalHandler.GetSignal(ASignal: string): TAppSignalStruct;
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.GetSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string'
    // 'TAppSignalStruct';
  end; // of function SetErrorParams: string

var
  i    : integer;
  found: boolean;
begin
  { function body }
  Result:= nil;
  i:= 0; found:= false;
  while (i < FSignalList.Count) and not(found) do begin
    found:= TAppSignalStruct(FSignalList[i]).Signal = ASignal;
    if not(found)
      then inc(i)
      else Result:= TAppSignalStruct(FSignalList[i]);
  end; // of while (i < FSignalList.Count) and not(found) do begin
end; // of TAppSignalHandler.GetSignal

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    LookForSignalSlot
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string; ASlot: TSignalSlot
  Result:    TSignalSlotStruct
-----------------------------------------------------------------------------}
function TAppSignalHandler.LookForSignalSlot
  (ASignal: string; AOwner: TObject): TSignalSlotStruct;
const
  cProcName = 'SignalHandler.TAppSignalHandler.LookForSignalSlot';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string; ASlot: TSignalSlot'
    // 'None';
  end; // of function SetErrorParams: string

var
  i    : integer;
  found: boolean;
begin
  { function body }
  Result:= nil; i:= 0; found:= false;
  while (i < FSignalSlotList.Count) and not(found) do begin
    found:= (TSignalSlotStruct(FSignalSlotList[i]).Signal = ASignal)
        and (TSignalSlotStruct(FSignalSlotList[i]).Receiver = AOwner);
    if not(found)
      then inc(i)
      else Result:= TSignalSlotStruct(FSignalSlotList[i]);
  end; // of while (i < FSignalSlotList.Count) and not(found) do begin
end; // of TAppSignalHandler.LookForSignalSlot

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    LookForSignalSlot
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string
  Result:    TSignalSlotStruct
-----------------------------------------------------------------------------}
function TAppSignalHandler.LookForSignalSlot
  (ASignal: string): TSignalSlotStruct;
const
  cProcName = 'SignalHandler.TAppSignalHandler.LookForSignalSlot';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string'
    // 'TSignalSlotStruct';
  end; // of function SetErrorParams: string

var
  i    : integer;
  found: boolean;
begin
  { function body }
  Result:= nil; i:= 0; found:= false;
  while (i < FSignalSlotList.Count) and not(found) do begin
    if (FSignalSlotList[i] <> nil) then begin
      found:= (TSignalSlotStruct(FSignalSlotList[i]).Signal = ASignal);
    end; // of if (FSignalSlotList[i] <> nil) then begin
    if not(found)
      then inc(i)
      else Result:= TSignalSlotStruct(FSignalSlotList[i]);
  end; // of while (i < FSignalSlotList.Count) and not(found) do begin
end; // of TAppSignalHandler.LookForSignalSlot

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    LookForSignal
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string; AOwner: TObject
  Result:    TAppSignalStruct
-----------------------------------------------------------------------------}
function TAppSignalHandler.LookForSignal
  (ASignal: string; AOwner: TObject): TAppSignalStruct;
const
  cProcName = 'SignalHandler.TAppSignalHandler.LookForSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string; AOwner: TObject'
    // 'TAppSignalStruct';
  end; // of function SetErrorParams: string

var
  i    : integer;
  found: boolean;
begin
  { function body }
  Result:= nil; i:= 0; found:= false;
  while(i < FSignalList.Count) and not(found) do begin
    found:= (TAppSignalStruct(FSignalList[i]).Signal = ASignal)
        and (TAppSignalStruct(FSignalList[i]).Sender = AOwner);
    if not(found)
      then inc(i)
      else Result:= TAppSignalStruct(FSignalList[i]);
  end; // of while(i < FSignalList.Count) and not(found) do begin
end; // of TAppSignalHandler.LookForSignal

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Methode:   EraseSignalSlots
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string
-----------------------------------------------------------------------------}
procedure TAppSignalHandler.EraseSignalSlots(ASignal: string);
const
  cProcName = 'SignalHandler.TAppSignalHandler.EraseSignalSlots';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string'
  end; // of function SetErrorParams: string

var
  i       : integer;
  found   : boolean;
  ThisSlot: TSignalSlotStruct;
begin
  { procedure body }
  i:= 0;
  while (i < FSignalSlotList.Count) do begin
    ThisSlot:= Self.LookForSignalSlot(ASignal);
    found:= ThisSlot <> nil;
    if found
      then FSignalSlotList.Items[FSignalSlotList.IndexOf(ThisSlot)]:= nil;
    inc(i);
  end; // of while (i < FSignalSlotList.Count) do begin
  FSignalSlotList.Pack;
  FSignalSlotList.Capacity:= FSignalSlotList.Count;
end; // of TAppSignalHandler.EraseSignalSlots

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Methode:   AddSignalSlot
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: ASignal: string; ASlot: TSignalSlot

  Registrieren der SignalSlots des Empfngers
-----------------------------------------------------------------------------}
procedure TAppSignalHandler.AddSignalSlot
  (ASignal: string; ASlot: TSignalSlotStruct);
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.AddSignalSlot';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: TSignalSlotStruct'
  end; // of function SetErrorParams: string

  function CheckParams: boolean;
  begin

  end; // of function CheckParams: boolean

var
  ThisSlot: TSignalSlotStruct;
begin
  { procedure body }
  if (ASignal <> '') then begin
    if (ASlot <> nil) then begin
      ThisSlot:= LookForSignalSlot(ASignal, ASlot.Receiver);
      if (ThisSlot = nil) then FSignalSlotList.Add(ASlot);
      FSignalSlotList.Pack;
      FSignalSlotList.Capacity:= FSignalSlotList.Count;
    end; // of if (ASlot <> nil) then begin
  end; // of if (ASignal <> '') then begin
end; // of TAppSignalHandler.AddSignalSlot

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Methode:   DelSignalSlot
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignalSlot: TSignalSlotStruct
-----------------------------------------------------------------------------}
procedure TAppSignalHandler.DelSignalSlot(ASignal: string; AOwner: TObject);
const
  cProcName = 'SignalHandler.TAppSignalHandler.DelSignalSlot';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignalSlot: TSignalSlotStruct'
  end; // of function SetErrorParams: string
var
  ThisSlot: TSignalSlotStruct;
begin
  { procedure body }
  if (ASignal <> '') and (AOwner <> nil) then begin
    ThisSlot:= LookForSignalSlot(ASignal, AOwner);
    if (ThisSlot <> nil) then begin
      FSignalSlotList.Delete(FSignalSlotList.IndexOf(ThisSlot));
      FSignalSlotList.Pack;
      FSignalSlotList.Capacity:= FSignalSlotList.Count;
    end; // of if (ThisSlot <> ni) then begin
  end; // of if (ASignal <> '') and (ASlot <> nil) then begin
end; // of TAppSignalHandler.DelSignalSlot

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    AddNewSignal
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: ASignal: TAppSignalStruct
  Result:    boolean
-----------------------------------------------------------------------------}
function TAppSignalHandler.AddNewSignal(ASignal: TAppSignalStruct): boolean;
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.AddNewSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: TAppSignalStruct'
    // 'boolean';
  end; // of function SetErrorParams: string

begin
  { function body }
  Result:= GetSignal(ASignal.Signal) = nil;
  if Result then begin
    { neues Signal hinzufgen }
    FSignalList.Add(ASignal);
  end; // if not(Result) then begin
end; // of TAppSignalHandler.AddNewSignal

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Methode:   DelSignal
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: ASignal: string; AOwner: TObject
-----------------------------------------------------------------------------}
procedure TAppSignalHandler.DelSignal(ASignal: string; AOwner: TObject);
const
  cProcName = 'SignalHandler.TAppSignalHandler.DelSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: string; AOwner: TObject'
  end; // of function SetErrorParams: string

var
  ThisSignal: TAppSignalStruct;
begin
  { procedure body }
  ThisSignal:= Self.LookForSignal(ASignal, AOwner);
  if (ThisSignal <> nil) then begin
    FSignalList.Delete(FSignalList.IndexOf(ThisSignal));
    FSignalList.Pack;
    FSignalList.Capacity:= FSignalList.Count;
    Self.EraseSignalSlots(ASignal);
  end; // of if (ThisSignal <> nil) then begin
end; // of TAppSignalHandler.DelSignal

{-----------------------------------------------------------------------------
  Class:     TAppSignalHandler
  Method:    SendSignal
  Author:    hardy
  Date:      10-Jun-2006
  Arguments: ASignal: TAppSignalStruct
  Result:    None
-----------------------------------------------------------------------------}
procedure TAppSignalHandler.SendSignal(ASignal: string);
const
  cProcName = 'AppSignalHandler.TAppSignalHandler.AddSignal';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASignal: TAppSignalStruct'
    // 'None';
  end; // of function SetErrorParams: string

var
  ThisSignal : TAppSignalStruct;
  ASignalSlot: TSignalSlotStruct;
  i: integer;
begin
  { function body }
  ThisSignal:= Self.GetSignal(ASignal);
  if (ThisSignal <> nil) then begin
    i:= 0;
    while (i < FSignalSlotList.Count) do begin
      ASignalSlot:= TSignalSlotStruct(FSignalSlotList[i]);
      if (ASignalSlot.Signal = ASignal) then begin
        if Assigned(ASignalSlot.SignalSlot)
          then ASignalSlot.SignalSlot(ThisSignal);
      end;
      inc(i);
    end; // of if (FSignalSlotList.Count > 0) then begin
  end; // of if (ThisSignal <> nil) then begin
end; // of TAppSignalHandler.SendSignal

{-----------------------------------------------------------------------------
 Unit Name: AppSignalHandler
 Author:    hardy
 Date:      10-Jun-2006
 Purpose:
 History:
-----------------------------------------------------------------------------}
initialization
  AppSignalHandler:= TAppSignalHandler.Create(nil);

{-----------------------------------------------------------------------------
 Unit Name: AppSignalHandler
 Author:    hardy
 Date:      10-Jun-2006
 Purpose:
 History:
-----------------------------------------------------------------------------}
finalization
  AppSignalHandler.Free;

end.










