{-----------------------------------------------------------------------------
 Unit Name: TimeTicker
 Author:    hardy
 Date:      09-Jun-2006
 Purpose:   Sendet TimerTicks an die Anwendung
   Diese Unit ist als 2te nach dem SignalHandler in der Uses-Anweisung des
   Projektes zu installieren. Gegebenenfalls mu dies von Hand gemacht werden:

   uses
     Forms, [QForms]
     SignalHandler,
     TimeTicker,
     

   Der TimeTicker kann an beliebiger Stelle im Programm auer Kraft gesetzt
   werden:

     AppTimeTicker.Enabled:= false;

   oder gestartet werden:

     AppTimeTicker.Enabled:= true;

   Die Signale mssen dem AppSignalHandler-Objekt bergeben werden
   damit sie wirksam werden.

   Die SignalSlot's knnen vom Empfnger im Konstruktor create oder
   an beliebiger Stelle des Codes initialisiert werden.

   Es knnen ein oder mehrere SignalSlot's im Objekt verwendet werden.
   Wenn Zwischenwerte gebraucht werden mssen diese mit einem Counter im
   jeweiligen Objekt gebildet werden.

 History:
-----------------------------------------------------------------------------}
unit TimeTicker;

{$MODE Delphi}

interface

uses
  SysUtils, Classes, SignalHandler, Controls;

type

  TCMTimeTicker = TMessage;

  TTimeTicker = class(TComponent)
  private
    { Private-Deklarationen }
    FTimer     : TTimer;
    F_5secTick : integer;
    F_10secTick: integer;
    F_20secTick: integer;
    F_30secTick: integer;
    F_1minTick : integer;
    F_5minTick : integer;
    F_10minTick: integer;
    F_20minTick: integer;
    F_30minTick: integer;
    F_1hourTick: integer;

    function  GetEnabled: boolean;
    procedure SetEnabled(Value: boolean);
    procedure InitSignals;
  protected
    { Protected-Deklarationen }

    procedure DoTimerTick(ASender: TObject);
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy;
  published
    { Published-Deklarationen }
    property Enabled: boolean read GetEnabled write SetEnabled;
  end; // of TTimeTicker

var
  AppTimeTicker: TTimeTicker;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Chat', [TTimeTicker]);
end;

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Methode:   Create
  Author:    hardy
  Date:      09-Jun-2006
  Arguments: AOwner: TComponent
-----------------------------------------------------------------------------}
constructor TTimeTicker.Create(AOwner: TComponent);
const
  cProcName = 'TimeTicker.TTimeTicker.Create';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'AOwner: TComponent'
  end; // of function SetErrorParams: string

begin
  { procedure body }
  inherited Create(AOwner);
  FTimer     := TTimer.Create(Self);
  FTimer.Enabled:= false;
  FTimer.OnTimer:= DoTimerTick;
  F_5secTick := 0;
  F_10secTick:= 0;
  F_20secTick:= 0;
  F_30secTick:= 0;
  F_1minTick := 0;
  F_5minTick := 0;
  F_10minTick:= 0;
  F_20minTick:= 0;
  F_30minTick:= 0;
  F_1hourTick:= 0;
  if (AppSignalHandler <> nil) then InitSignals;
end; // of TTimeTicker.Create

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Methode:   InitSignals
  Author:    hardy
  Date:      11-Jun-2006
  Arguments: None
-----------------------------------------------------------------------------}
procedure TTimeTicker.InitSignals;
const
  cProcName = 'TimeTicker.TTimeTicker.InitSignals';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'None'
  end; // of function SetErrorParams: string

var
  ThisSignal: TAppSignalStruct;
begin
  { procedure body }
  ThisSignal:= SignalHandler.CreateSignal('CM_1secTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_5secTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_10secTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_20secTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_30secTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_1minTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_5minTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_10minTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_20minTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_30minTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
  ThisSignal:= SignalHandler.CreateSignal('CM_1hourTick', Self, nil);
  if (ThisSignal <> nil) then AppSignalHandler.AddNewSignal(ThisSignal);
end; // of TTimeTicker.InitSignals

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Methode:   Destroy
  Author:    hardy
  Date:      09-Jun-2006
  Arguments: None
-----------------------------------------------------------------------------}
destructor TTimeTicker.Destroy;
const
  cProcName = 'TimeTicker.TTimeTicker.Destroy';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'None'
  end; // of function SetErrorParams: string

begin
  { procedure body }
  FTimer.Free;
  AppSignalHandler.DelSignal('CM_1secTick', Self);
  AppSignalHandler.DelSignal('CM_5secTick', Self);
  AppSignalHandler.DelSignal('CM_10secTick', Self);
  AppSignalHandler.DelSignal('CM_20secTick', Self);
  AppSignalHandler.DelSignal('CM_30secTick', Self);
  AppSignalHandler.DelSignal('CM_1minTick', Self);
  AppSignalHandler.DelSignal('CM_5minTick', Self);
  AppSignalHandler.DelSignal('CM_10minTick', Self);
  AppSignalHandler.DelSignal('CM_20minTick', Self);
  AppSignalHandler.DelSignal('CM_30minTick', Self);
  AppSignalHandler.DelSignal('CM_1hourTick', Self);
end; // of TTimeTicker.Destroy

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Method:    GetEnabled
  Author:    hardy
  Date:      09-Jun-2006
  Arguments: None
  Result:    boolean
-----------------------------------------------------------------------------}
function TTimeTicker.GetEnabled: boolean;
const
  cProcName = 'TimeTicker.TTimeTicker.GetEnabled';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'None'
    // 'boolean';
  end; // of function SetErrorParams: string

begin
  { function body }
  Result:= FTimer.Enabled;
end; // of TTimeTicker.GetEnabled

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Methode:   SetEnabled
  Author:    hardy
  Date:      09-Jun-2006
  Arguments: Value: boolean
-----------------------------------------------------------------------------}
procedure TTimeTicker.SetEnabled(Value: boolean);
const
  cProcName = 'TimeTicker.TTimeTicker.SetEnabled';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'Value: boolean'
  end; // of function SetErrorParams: string

begin
  { procedure body }
  if (Value <> FTimer.Enabled) then FTimer.Enabled:= Value;
end; // of TTimeTicker.SetEnabled

{-----------------------------------------------------------------------------
  Class:     TTimeTicker
  Methode:   DoTimerTick
  Author:    hardy
  Date:      09-Jun-2006
  Arguments: ASender: TObject
-----------------------------------------------------------------------------}
procedure TTimeTicker.DoTimerTick(ASender: TObject);
const
  cProcName = 'TimeTicker.TTimeTicker.DoTimerTick';

  function SetErrorParams: string;
  begin
    Result:= '';
    // 'ASender: TObject'
  end; // of function SetErrorParams: string

var
  ThisMsg: TCMTimeTicker;
begin
  { procedure body }
  FillChar(ThisMsg, SizeOf(ThisMsg), #0);
  AppSignalHandler.SendSignal('CM_1secTick');
  inc(F_5secTick);
  inc(F_10secTick);
  inc(F_20secTick);
  inc(F_30secTick);
  if (F_5secTick = 5) then begin
    F_5secTick:= 0;
    AppSignalHandler.SendSignal('CM_5secTick');
  end; // of if (F_5secTick = 5) then begin
  if (F_10secTick = 10) then begin
    F_10secTick:= 0;
    AppSignalHandler.SendSignal('CM_10secTick');
  end; // of if (F_10secTick = 10) then begin
  if (F_20secTick = 20) then begin
    F_20secTick:= 0;
    AppSignalHandler.SendSignal('CM_20secTick');
  end; // of if (F_20secTick = 20) then begin
  if (F_30secTick = 30) then begin
    inc(F_1minTick);
    F_30secTick:= 0;
    AppSignalHandler.SendSignal('CM_30secTick');
  end; // of if (F_30secTick = 30) then begin
  if (F_1minTick = 2) then begin
    inc(F_5minTick);
    inc(F_10minTick);
    inc(F_20minTick);
    inc(F_30minTick);
    inc(F_1hourTick);
    F_1minTick:= 0;
    AppSignalHandler.SendSignal('CM_1minTick');
  end; // of if (F_1minTick = 2) then begin
  if (F_5minTick = 5) then begin
    F_5minTick:= 0;
    AppSignalHandler.SendSignal('CM_5minTick');
  end; // of if (F_5minTick = 5) then begin
  if (F_10minTick = 10) then begin
    F_10minTick:= 0;
    AppSignalHandler.SendSignal('CM_10minTick');
  end; // of if (F_10minTick = 10) then begin
  if (F_20minTick = 20) then begin
    F_20minTick:= 0;
    AppSignalHandler.SendSignal('CM_20minTick');
  end; // of if (F_20minTick = 20) then begin
  if (F_30minTick = 30) then begin
    F_30minTick:= 0;
    AppSignalHandler.SendSignal('CM_30minTick');
  end; // of if (F_30minTick = 30) then begin
  if (F_1hourTick = 60) then begin
    F_1hourTick:= 0;
    AppSignalHandler.SendSignal('CM_1hourTick');
  end; // of if (F_1hourTick = 60) then begin
end; // of TTimeTicker.DoTimerTick

{-----------------------------------------------------------------------------
 Unit Name: TimeTicker
 Author:    hardy
 Date:      10-Jun-2006
 Purpose:
 History:
-----------------------------------------------------------------------------}
initialization
  AppTimeTicker:= TTimeTicker.Create(nil);

{-----------------------------------------------------------------------------
 Unit Name: TimeTicker
 Author:    hardy
 Date:      11-Jun-2006
 Purpose:
 History:
-----------------------------------------------------------------------------}
finalization
  AppTimeTicker.Free;

end.
